<?php

namespace app\admin\service\notify;

use app\model\Goods;
use app\model\MemberAccount;
use app\model\MemberQuota;
use app\model\MemberRole;
use app\model\Order;
use app\model\OrderSheet;
use app\model\RuleAdded;
use app\model\RuleAddedComponent;
use support\exception\BusinessException;

class RechargeService
{
    /**
     * @Desc
     * @Author Gorden
     * @Date 2024/5/28 16:50
     *
     * @param $orderId
     * @return mixed
     * @throws BusinessException
     */
    public static function disposeOrder($orderId)
    {
        $orderSheet = OrderSheet::where('join_sheet_order_id', $orderId)->first();
        if (!$orderSheet) {
            throw new BusinessException('订单数据异常');
        }
        $goods = Goods::find($orderSheet->join_sheet_goods_id);
        if (empty($goods->goods_attribute_json)) {
            throw new BusinessException('产品数据异常');
        }
        $attributeJson = json_decode($goods->goods_attribute_json, true);
        if (!isset($attributeJson['added'])) {
            throw new BusinessException('产品数据异常');
        }

        return $attributeJson['added']['nbr'];
    }

    /**
     * @Desc
     * @Author Gorden
     * @Date 2024/5/28 17:23
     *
     * @param $memberId
     * @return mixed|void
     */
    public static function disposeRole($memberId, $payAmount)
    {
        $memberAccount = MemberAccount::where('join_account_member_id', $memberId)->first();
        if ($memberAccount->member_account_income > $memberAccount->member_account_expend) {
            $money = $memberAccount->member_account_incom;
        } else {
            $money = $memberAccount->member_account_expend;
        }

        $memberRoles = MemberRole::where('member_role_status', 'ACTIVED')->get()->toArray();
        foreach ($memberRoles as $role) {
            $amount = floatval($money) + floatval($payAmount);
            if ($amount >= floatval($role['member_role_range_begin']) && $amount < floatval($role['member_role_range_end'])) {
                return $role['member_role_id'];
            }
        }

        return false;
    }

    /**
     * @Desc
     * @Author Gorden
     * @Date 2024/5/28 18:03
     *
     * @param $roleId
     * @param $memberId
     * @return void
     * @throws BusinessException
     */
    public static function disposeAdded($roleId, $memberId)
    {
        $role = MemberRole::find($roleId);
        if (empty($role->member_role_rule_json)) {
            throw new BusinessException('增值套包数据异常');
        }
        $ruleJson = json_decode($role->member_role_rule_json, true);
        if (!isset($ruleJson['added'])) {
            throw new BusinessException('增值套包数据异常');
        }
        $addedId = $ruleJson['added'];
        $added = RuleAdded::find($addedId);
        $diff = $added->rule_added_goods_diff;

        $components = RuleAddedComponent::where('join_component_rule_added_id', $addedId)->get()->toArray();
        switch ($diff) {
            case "SUM":
                self::diffSum($memberId, $components);
                break;
            case "REPLACE":
                self::diffReplace($memberId, $components);
                break;
            case "NONE":
                self::diffNone($memberId, $components);
        }
    }

    /**
     * @Desc
     * @Author Gorden
     * @Date 2024/5/28 18:03
     *
     * @param $memberId
     * @param $components
     * @return void
     */
    public static function diffSum($memberId, $components)
    {
        foreach ($components as $component) {
            self::saveMemberQuota($memberId, $component);
        }
    }

    /**
     * @Desc
     * @Author Gorden
     * @Date 2024/5/28 18:02
     *
     * @param $memberId
     * @param $components
     * @return void
     */
    public static function diffReplace($memberId, $components)
    {
        foreach ($components as $component) {
            if (MemberQuota::where('join_quota_member_id', $memberId)->where('join_member_rule_added_component_id', $component['rule_added_component_id'])->exists()) {
                MemberQuota::where('join_quota_member_id', $memberId)
                    ->where('join_member_rule_added_component_id', $component['rule_added_component_id'])
                    ->where('member_quota_status', 'PENDING')
                    ->delete();
            }

            self::saveMemberQuota($memberId, $component);
        }

    }

    /**
     * @Desc
     * @Author Gorden
     * @Date 2024/5/28 18:02
     *
     * @param $memberId
     * @param $components
     * @return void
     */
    public static function diffNone($memberId, $components)
    {
        foreach ($components as $component) {
            if (MemberQuota::where('join_quota_member_id', $memberId)->where('join_member_rule_added_component_id', $component['rule_added_component_id'])->exists()) {
                continue;
            }

            self::saveMemberQuota($memberId, $component);
        }
    }

    /**
     * @Desc
     * @Author Gorden
     * @Date 2024/5/28 18:02
     *
     * @param $memberId
     * @param $component
     * @return void
     */
    private static function saveMemberQuota($memberId, $component)
    {
        for ($i = 0; $i < floatval($component['rule_added_component_goods_nbr']); $i++) {
            $model = new MemberQuota();
            $model->member_quota_id = 'MQ' . date('YmdHis') . random_string(6, 'up');
            $model->join_quota_member_id = $memberId;
            $model->join_member_rule_added_component_id = $component['rule_added_component_id'];
            $model->member_quota_status = 'PENDING';
            $model->member_quota_category = $component['rule_added_component_category'];
            $model->member_quota_name = $component['rule_added_component_name'];
            $model->member_quota_nbr = $component['rule_added_component_goods_nbr'];
            $model->member_quota_json = $component['rule_added_component_goods_json'];
            $model->member_quota_addtimes = time();
            $model->save();
        }
    }
}